﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ServiceModel;
using System.Threading;
using System.Collections.ObjectModel;

namespace PhotoGallery1
{
	public partial class Page : UserControl
	{
		#region Object Definitions

		public class ImageInfo
		{
			public enum ImageTypeOptions { Image, Folder };
			public ImageTypeOptions ImageType { get; set; }
			public string RelativePath { get; set; }
			public enum RotationOptions { None, Ninety, OneEighty, TwoSeventy };
			public RotationOptions Rotation { get; set; }
			public string Privacy { get; set; }
		}

		#endregion

		#region Private Members

		private ObservableCollection<Object> _thumbs = new ObservableCollection<object>();

		private ImageSource _img = null;

		private Storyboard _fadeOutImage = null;
		private Storyboard _fadeInImage = null;
		private Storyboard _slideShowTimer = null;

		private string _currentFolder = null;

		private bool _isPlaying = false;
		private int _selectedIndex = -1;

		private bool _isAdmin = false;

		#endregion

		#region Service Proxy Stuff

		private ServiceLayerProxy.ServiceLayerSoapClient _proxy = null;

		private void CreateProxy()
		{
			// Create the proxy.
			//
			BasicHttpBinding bind = new BasicHttpBinding();
			string appUri = System.Windows.Application.Current.Host.Source.AbsoluteUri;
			int posLastSlash = appUri.LastIndexOf("/");
			string webUri = appUri.Substring(0, posLastSlash + 1);
			string serviceUri = webUri + "ServiceLayer.asmx";
			EndpointAddress endpoint = new EndpointAddress(serviceUri);
			_proxy = new PhotoGallery1.ServiceLayerProxy.ServiceLayerSoapClient(bind, endpoint);

			// Connect up service proxy events.
			//
			_proxy.GetThumbListCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.GetThumbListCompletedEventArgs>(_proxy_GetThumbListCompleted);
			_proxy.GetImageInfoCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.GetImageInfoCompletedEventArgs>(_proxy_GetImageInfoCompleted);
			_proxy.LoginCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.LoginCompletedEventArgs>(_proxy_LoginCompleted);
			_proxy.RotateCCWCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.RotateCCWCompletedEventArgs>(_proxy_RotateCCWCompleted);
			_proxy.RotateCWCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.RotateCWCompletedEventArgs>(_proxy_RotateCWCompleted);
			_proxy.SetPrivacyFamilyCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.SetPrivacyFamilyCompletedEventArgs>(_proxy_SetPrivacyFamilyCompleted);
			_proxy.SetPrivacyPrivateCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.SetPrivacyPrivateCompletedEventArgs>(_proxy_SetPrivacyPrivateCompleted);
			_proxy.SetPrivacyPublicCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.SetPrivacyPublicCompletedEventArgs>(_proxy_SetPrivacyPublicCompleted);
			_proxy.DoSearchCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.DoSearchCompletedEventArgs>(_proxy_DoSearchCompleted);
			_proxy.SetTitleCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.SetTitleCompletedEventArgs>(_proxy_SetTitleCompleted);
			_proxy.SetTagsCompleted += new EventHandler<PhotoGallery1.ServiceLayerProxy.SetTagsCompletedEventArgs>(_proxy_SetTagsCompleted);

			// Provide the proxy to the login user control
			//
			LoginPanel.Proxy = _proxy;
		}

		void _proxy_SetTagsCompleted(object sender, PhotoGallery1.ServiceLayerProxy.SetTagsCompletedEventArgs e)
		{
			ProcessImageInfo(e.Result);
		}

		void _proxy_SetTitleCompleted(object sender, PhotoGallery1.ServiceLayerProxy.SetTitleCompletedEventArgs e)
		{
			ProcessImageInfo(e.Result);
		}

		void _proxy_DoSearchCompleted(object sender, PhotoGallery1.ServiceLayerProxy.DoSearchCompletedEventArgs e)
		{
			ProcessThumbListString(e.Result);
			BackToRootButton.IsEnabled = true;
		}

		void _proxy_SetPrivacyPublicCompleted(object sender, PhotoGallery1.ServiceLayerProxy.SetPrivacyPublicCompletedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				thumb.Privacy = "Public";
			}
		}

		void _proxy_SetPrivacyPrivateCompleted(object sender, PhotoGallery1.ServiceLayerProxy.SetPrivacyPrivateCompletedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				thumb.Privacy = "Private";
			}
		}

		void _proxy_SetPrivacyFamilyCompleted(object sender, PhotoGallery1.ServiceLayerProxy.SetPrivacyFamilyCompletedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				thumb.Privacy = "Family";
			}
		}

		void _proxy_RotateCWCompleted(object sender, PhotoGallery1.ServiceLayerProxy.RotateCWCompletedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				thumb.Rotation++;
				if (thumb.Rotation > 3)
				{
					thumb.Rotation = 0;
				}
			}
			ShowImage(thumb);
		}

		void _proxy_RotateCCWCompleted(object sender, PhotoGallery1.ServiceLayerProxy.RotateCCWCompletedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				thumb.Rotation--;
				if (thumb.Rotation < 0)
				{
					thumb.Rotation = 3;
				}
			}
			ShowImage(thumb);
		}

		void _proxy_LoginCompleted(object sender, PhotoGallery1.ServiceLayerProxy.LoginCompletedEventArgs e)
		{
			if (e.Result == "Public")
			{
				// Login attempt was not successful.
				//

			}
			else
			{
				// Login attempt was successful.
				//
				LoginPanel.Visibility = Visibility.Collapsed;
				LogInButton.Visibility = Visibility.Collapsed;
				LogOutButton.Visibility = Visibility.Visible;

				if (e.Result == "Admin")
				{
					PrivacyOption.Visibility = Visibility.Visible;
					RotateCCW.Visibility = Visibility.Visible;
					RotateCW.Visibility = Visibility.Visible;
					InfoAreaStatic.Visibility = Visibility.Collapsed;
					InfoAreaEditable.Visibility = Visibility.Visible;
					
					_isAdmin = true;
				}
			}
		}

		void _proxy_GetThumbListCompleted(object sender, PhotoGallery1.ServiceLayerProxy.GetThumbListCompletedEventArgs e)
		{
			ProcessThumbListString(e.Result);
		}

		void _proxy_GetImageInfoCompleted(object sender, PhotoGallery1.ServiceLayerProxy.GetImageInfoCompletedEventArgs e)
		{
			ProcessImageInfo(e.Result);
		}

		private void ProcessImageInfo(string info)
		{
			// Expecting string in the following format.
			//	Title [tag1,tag2,...tagN]		(non-admins)
			//	Title [tag1,tag2,...tagN] - Privacy, Rotation	(admins)
			//

			// If this is an admin, strip off the admin info.
			//
			if (_isAdmin)
			{
				int nPosDash = info.LastIndexOf("-");
				if (nPosDash != -1)
				{
					string adminInfo = info.Substring(nPosDash + 1);
					info = info.Substring(0, nPosDash);

					string[] parts = adminInfo.Split(',');
					if (parts.Length == 2)
					{
						string privacy = parts[0].Trim();
						string rotation = parts[1].Trim();
						PrivacyOption.SelectedItem = privacy;
						for (int i = 0; i < PrivacyOption.Items.Count; i++)
						{
							ComboBoxItem obj = PrivacyOption.Items[i] as ComboBoxItem;
							if (obj.Content.ToString() == privacy)
							{
								obj.IsSelected = true;
								//PrivacyOption.SelectedItem = obj;
								PrivacyOption.SelectedIndex = i;
							}
						}
					}
				}
			}

			// Strip off title and tags.
			//
			int nPosLeftBracket = info.LastIndexOf("[");
			int nPosRightBracket = info.LastIndexOf("]");
			if (nPosLeftBracket != -1 && nPosRightBracket != -1 && nPosLeftBracket < nPosRightBracket)
			{
				Title.Text = info.Substring(0, nPosLeftBracket).Trim();
				Tags.Text = info.Substring(nPosLeftBracket + 1, nPosRightBracket - nPosLeftBracket - 1).Trim();
				info = info.Substring(nPosRightBracket + 1);
			}
			else
			{
				Title.Text = info.Trim();
				Tags.Text = string.Empty;
			}

			// Set the editable version of this info.
			//
			TitleEditable.Text = Title.Text;
			TagsEditable.Text = Tags.Text;
		}

		#endregion

		#region Private Methods

		private void ProcessThumbListString(string thumbList)
		{
			// Got a new thumbnail list
			//
			string[] imageData = thumbList.Split('|');
			List<ImageInfo> imgs = new List<ImageInfo>();
			foreach (string imgInfo in imageData)
			{
				string[] data = imgInfo.Split(':');
				ImageInfo img = new ImageInfo();
				if (data[0] == "d")
				{
					img.ImageType = ImageInfo.ImageTypeOptions.Folder;
				}
				else
				{
					img.ImageType = ImageInfo.ImageTypeOptions.Image;
				}
				img.RelativePath = data[1].Replace("\\", "/");
				img.Rotation = (ImageInfo.RotationOptions)int.Parse(data[2]);
				img.Privacy = data[3];

				imgs.Add(img);
			}

			LoadImageStrip(imgs);
		}

		private void FetchImgList(string relativePath)
		{
			_currentFolder = relativePath;
			if (string.IsNullOrEmpty(_currentFolder))
			{
				BackOneButton.IsEnabled = false;
				BackToRootButton.IsEnabled = false;
			}
			else
			{
				BackToRootButton.IsEnabled = true;
				BackOneButton.IsEnabled = true;
			}
			_proxy.GetThumbListAsync(relativePath);
		}

		private string GetAbsoluteUrl(string relPath)
		{
			if (string.IsNullOrEmpty(relPath))
			{
				return relPath;
			}

            string strFullUrl;
			if (relPath.StartsWith("http:", StringComparison.OrdinalIgnoreCase)
			  || relPath.StartsWith("https:", StringComparison.OrdinalIgnoreCase)
			  || relPath.StartsWith("file:", StringComparison.OrdinalIgnoreCase)
              )
            {
                //already absolute
				strFullUrl = relPath;
            }
            else
            {
                //relative, need to convert to absolute
                strFullUrl = System.Windows.Application.Current.Host.Source.AbsoluteUri;
				if (strFullUrl.IndexOf("ClientBin") > 0)
				{
					strFullUrl = strFullUrl.Substring(0, strFullUrl.IndexOf("ClientBin")) + relPath;
				}
				else
				{
					strFullUrl = strFullUrl.Substring(0, strFullUrl.LastIndexOf("/") + 1) + relPath;
				}
            }

            return strFullUrl;
        }

		private void LoadImageStrip(List<ImageInfo> imgs)
		{
			// Clear the current images.
			//
			//ImageStrip.Items.Clear();
			_thumbs.Clear();
			
			// Reset the slide show
			//
			_isPlaying = false;
			_slideShowTimer.Stop();
			StopButton.Visibility = Visibility.Collapsed;
			PlayButton.Visibility = Visibility.Visible;
			PlayButton.IsEnabled = false;

			// Add each image.
			//
			bool foundImage = false;
			foreach (ImageInfo img in imgs)
			{
				// Create a URL that can be used to dynamically
				//	fetch the image via the image handler.
				//
				string relPath = "Image.ashx?rp=" + img.RelativePath + "&t=1";
				string absPath = GetAbsoluteUrl(relPath);

				if (img.ImageType == ImageInfo.ImageTypeOptions.Folder)
				{
					// This is a folder of images.
					//
					ImageFolder folder = new ImageFolder(absPath);
					folder.RelativePath = img.RelativePath.Replace("folderthumb", "");
					_thumbs.Add(folder);
				}
				else
				{
					// This is an image.
					//
					Thumbnail thumb = new Thumbnail(absPath);
					thumb.RelativePath = img.RelativePath;
					thumb.Rotation = (int)img.Rotation;
					thumb.Privacy = img.Privacy;
					_thumbs.Add(thumb);

					if (!foundImage)
					{
						ShowImage(thumb);
						foundImage = true;
						PlayButton.IsEnabled = true;
					}
				}
			}

			_selectedIndex = GetSelectedIndex();
		}

		private void ShowImage(Thumbnail thumb)
		{
			ImageStrip.SelectedItem = thumb;

			string relativePath = thumb.RelativePath;
			string relUrl = "Image.ashx?rp=" + relativePath;
			if (thumb.Rotation != 0)
			{
				string rotation = string.Format("&r={0}", thumb.Rotation);
				relUrl += rotation;
			}
			string absUrl = GetAbsoluteUrl(relUrl);

			Uri uri = new Uri(absUrl, UriKind.RelativeOrAbsolute);
			_img = new System.Windows.Media.Imaging.BitmapImage(uri);

			string downloadUrl = absUrl + "&d=1";
			Uri download = new Uri(downloadUrl, UriKind.RelativeOrAbsolute);
			SaveLink.NavigateUri = download;

			Thumbnail currentThumb = GetSelectedThumbnail();
			if (currentThumb != null)
			{
				currentThumb.DeSelect();
			}
			_fadeOutImage.Begin();

			thumb.Select();
			_proxy.GetImageInfoAsync(absUrl);
		}

		private void ShowFolder(object sender, EventArgs e)
		{
			ImageFolder folder = sender as ImageFolder;
			string relativePath = folder.RelativePath;

			FetchImgList(relativePath);
		}

		private void FadeOutAnim_Completed(object sender, EventArgs e)
		{
			if (_img == null)
			{
				return;
			}
			BigImage.SetValue(Image.SourceProperty, _img);

			_fadeInImage.Begin();
		}

		private int GetSelectedIndex()
		{
			var thumbs = ImageStrip.Items.Where(o => o.GetType() == typeof(Thumbnail)).ToList();
			int selectedIndex = -1;
			for (int i = 0; i < thumbs.Count; i++)
			{
				Thumbnail thumb = thumbs[i] as Thumbnail;
				if (thumb != null && thumb.IsSelected)
				{
					selectedIndex = i;
					break;
				}
			}

			return selectedIndex;
		}

		private Thumbnail GetSelectedThumbnail()
		{
			var thumbs = ImageStrip.Items.Where(o => o.GetType() == typeof(Thumbnail)).ToList();
			for (int i = 0; i < thumbs.Count; i++)
			{
				Thumbnail thumb = thumbs[i] as Thumbnail;
				if (thumb != null && thumb.IsSelected)
				{
					return thumb;
				}
			}

			return null;
		}

		#region Slide Show Helpers

		private void StartSlideShow()
		{
			if (!_isPlaying)
			{
				_isPlaying = true;

				_slideShowTimer.Begin();
			}
		}

		private void SlideShowTimer_Completed(object sender, EventArgs e)
		{
			if (_isPlaying)
			{
				// Get the next thumbnail in the slideshow.
				//
				Thumbnail thumb = GetNextThumbnail();

				// Select the image, scroll to make it visible,
				//	and show the image in the big image.
				//
				SelectScrollAndShow(thumb);

				// Start the timer for the next image.
				//
				_slideShowTimer.Begin();
			}
		}

		private Thumbnail GetNextThumbnail()
		{
			var thumbs = ImageStrip.Items.Where(o => o.GetType() == typeof(Thumbnail)).ToList();
			int newIndex = GetSelectedIndex();
			Thumbnail thumb = null;
			while (thumb == null)
			{
				newIndex++;
				if (newIndex >= thumbs.Count)
				{
					newIndex = 0;
				}
				thumb = thumbs[newIndex] as Thumbnail;
			}
			return thumb;
		}

		private void SelectScrollAndShow(Thumbnail thumb)
		{
			thumb.Select();
			ImageStrip.SelectedItem = thumb;

			ScrollViewer imageScroll = FindImageStripScrollViewer();
			if (imageScroll != null)
			{
				double height = thumb.ActualHeight;
				double scrollPos = ImageStrip.SelectedIndex * height;
				imageScroll.ScrollToVerticalOffset(scrollPos);
			}


			ShowImage(thumb);
		}

		private ScrollViewer FindImageStripScrollViewer()
		{
			Border scrollBorder = VisualTreeHelper.GetChild(ImageStrip, 0) as Border;
			if (scrollBorder is Border)
			{
				ScrollViewer scroll = scrollBorder.Child as ScrollViewer;
				if (scroll is ScrollViewer)
				{
					return scroll;
				}
				else
				{
					return null;
				}
			}
			else
			{
				return null;
			}
		}

		#endregion

		private void SetupImageAnimation()
		{
			// Get handles to the image animations.
			//
			_fadeInImage = (Storyboard)this.FindName("FadeInImage");
			_fadeOutImage = (Storyboard)this.FindName("FadeOutImage");
			_slideShowTimer = (Storyboard)this.FindName("SlideShowTimer");

			// Attach an event to the image fade out animation.
			//
			_fadeOutImage.Completed += new EventHandler(FadeOutAnim_Completed);
			_slideShowTimer.Completed += new EventHandler(SlideShowTimer_Completed);
		}

		#endregion

		#region UI Handlers

		private void ImageStrip_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			if (ImageStrip.SelectedItem is Thumbnail)
			{
				_slideShowTimer.Stop();
				Thumbnail thumb = ImageStrip.SelectedItem as Thumbnail;
				ShowImage(thumb);
			}
			else if (ImageStrip.SelectedItem is ImageFolder)
			{
				_slideShowTimer.Stop();
				ImageFolder folder = ImageStrip.SelectedItem as ImageFolder;
				string relativePath = folder.RelativePath;

				FetchImgList(relativePath);
			}
		}

		private void Login_Click(object sender, RoutedEventArgs e)
		{
			LoginPanel.Visibility = Visibility.Visible;
		}

		private void Logout_Click(object sender, RoutedEventArgs e)
		{
			_isAdmin = false;
			_proxy.LogoutAsync();
			PrivacyOption.Visibility = Visibility.Collapsed;
			RotateCCW.Visibility = Visibility.Collapsed;
			RotateCW.Visibility = Visibility.Collapsed;
			LogOutButton.Visibility = Visibility.Collapsed;
			LogInButton.Visibility = Visibility.Visible;
			InfoAreaEditable.Visibility = Visibility.Collapsed;
			InfoAreaStatic.Visibility = Visibility.Visible;
		}

		private void RotateCW_Click(object sender, RoutedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				_proxy.RotateCWAsync(thumb.ImageSource);
			}
		}

		private void RotateCCW_Click(object sender, RoutedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				_proxy.RotateCCWAsync(thumb.ImageSource);
			}
		}

		private void PrivacyOption_Changed(object sender, SelectionChangedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if(thumb!=null)
			{
				for (int i = 0; i < PrivacyOption.Items.Count; i++)
				{
					ComboBoxItem cbi = PrivacyOption.Items[i] as ComboBoxItem;
					if (cbi.IsSelected)
					{
						string value = cbi.Content.ToString();
						if (thumb.Privacy != value)
						{
							if (value == "Private")
							{
								_proxy.SetPrivacyPrivateAsync(thumb.ImageSource);
							}
							else if (value == "Family")
							{
								_proxy.SetPrivacyFamilyAsync(thumb.ImageSource);
							}
							else if (value == "Public")
							{
								_proxy.SetPrivacyPublicAsync(thumb.ImageSource);
							}
						}
					}
				}
			}
		}

		private void Play_Click(object sender, RoutedEventArgs e)
		{
			PlayButton.Visibility = Visibility.Collapsed;
			StopButton.Visibility = Visibility.Visible;

			StartSlideShow();
		}

		private void Stop_Click(object sender, RoutedEventArgs e)
		{
			StopButton.Visibility = Visibility.Collapsed;
			PlayButton.Visibility = Visibility.Visible;

			_slideShowTimer.Stop();
			_isPlaying = false;
		}

		private void BackToRoot_Click(object sender, RoutedEventArgs e)
		{
			FetchImgList(string.Empty);
		}

		private void BackOneFolder_Click(object sender, RoutedEventArgs e)
		{
			string newFolder = _currentFolder;
			if (newFolder[newFolder.Length - 1] == '/' || newFolder[newFolder.Length - 1] == '\\')
			{
				newFolder = newFolder.Substring(0, newFolder.Length - 1);
			}
			int last = newFolder.LastIndexOf('/');
			if (last == -1)
			{
				last = _currentFolder.LastIndexOf('\\');
			}
			if (last == -1)
			{
				last = 0;
			}
			newFolder = newFolder.Substring(0, last);

			FetchImgList(newFolder);
		}

		private void Search_Click(object sender, RoutedEventArgs e)
		{
			string searchText = SearchText.Text;
			if (!string.IsNullOrEmpty(searchText))
			{
				_proxy.DoSearchAsync(searchText);
			}
			else
			{
				FetchImgList(_currentFolder);
			}
		}

		private void Title_LostFocus(object sender, RoutedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				if (!Title.Text.Equals(TitleEditable.Text, StringComparison.CurrentCultureIgnoreCase))
				{
					_proxy.SetTitleAsync(thumb.ImageSource, TitleEditable.Text);
				}
			}
		}

		private void Tags_LostFocus(object sender, RoutedEventArgs e)
		{
			Thumbnail thumb = GetSelectedThumbnail();
			if (thumb != null)
			{
				if (!Tags.Text.Equals(TagsEditable.Text, StringComparison.CurrentCultureIgnoreCase))
				{
					_proxy.SetTagsAsync(thumb.ImageSource, TagsEditable.Text);
				}
			}
		}

		#endregion

		public Page()
		{
			InitializeComponent();

			// Create the service proxy.
			//
			CreateProxy();

			// Setup for callbacks used during
			// image animations.
			//
			SetupImageAnimation();

			// Bind the list to the observable collection
			//
			ImageStrip.ItemsSource = _thumbs;

			// Fetch the image list.
			//
			FetchImgList(string.Empty);
		}
	}
}
