﻿<%@ WebHandler Language="C#" Class="Image" %>

using System;
using System.Web;
using Rlc.PhotoGallery.Bll;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Text;

public class Image : IHttpHandler 
{
    public void ProcessRequest (HttpContext context)
	{
		// Get the image relative path.
		//
		string imageRelativePath = context.Request.Params["rp"];
		if (string.IsNullOrEmpty(imageRelativePath))
		{
			return;
		}
		string imageFullPath = Path.Combine(ConfigData.RootGalleryDirectory, imageRelativePath);
		bool isThumb = false;
		if (context.Request.Params["t"] != null)
		{
			isThumb = true;
		}
		if (isThumb)
		{
			imageFullPath = GalleryManager.GetThumbNailImageFullPath(imageFullPath, 0, ConfigData.RootThumbDirectory, ConfigData.RootGalleryDirectory);
		}
		bool isDownload = false;
		if (context.Request.Params["d"] != null)
		{
			isDownload = true;
		}
		int rotationIndex = 0;
		if (context.Request.Params["r"] != null)
		{
			int.TryParse(context.Request.Params["r"], out rotationIndex);
		}

		if (string.IsNullOrEmpty(imageFullPath))
		{
			return;
		}
		if (!File.Exists(imageFullPath))
		{
			return;
		}

		System.Drawing.Image image = null;
		try
		{
			image = System.Drawing.Image.FromFile(imageFullPath);
		}
		catch (Exception ex)
		{
		}
		if (image == null)
		{
			CreateWordImage("no image!", context);
			return;
		}
		
		// Do the rotation if it is not a thumbnail.
		//	Thumbnails were created already rotated.
		//
		if (!isThumb)
		{
			if (rotationIndex == 1)
			{
				image.RotateFlip(RotateFlipType.Rotate90FlipNone);
			}
			else if (rotationIndex == 2)
			{
				image.RotateFlip(RotateFlipType.Rotate180FlipNone);
			}
			else if (rotationIndex == 3)
			{
				image.RotateFlip(RotateFlipType.Rotate270FlipNone);
			}
		}
		
		if (isDownload)
		{
			context.Response.AddHeader("Content-Disposition", "attachment; filename=" + Path.GetFileName(imageFullPath));
		}
		context.Response.ContentType = GetContentType(image);
		image.Save(context.Response.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);
		context.Response.End();
	}
	
	public string GetContentType(System.Drawing.Image image)
	{
		if (image.RawFormat == ImageFormat.Jpeg)
		{
			return "Image/jpeg";
		}
		else if (image.RawFormat == ImageFormat.Png)
		{
			return "Image/png";
		}
		else if (image.RawFormat == ImageFormat.Bmp)
		{
			return "Image/bmp";
		}
		else if (image.RawFormat == ImageFormat.Gif)
		{
			return "Image/gif";
		}
		else
		{
			return "Image/jpeg";
		}
	}
 
    public bool IsReusable 
	{
        get 
		{
            return false;
        }
    }
	
	private void CreateWordImage(string text, HttpContext context)
	{
		int textLength = text.Length;
		int fontSize = 12;

		// Set canvas width & height
		int width = (fontSize * textLength) - ((textLength * fontSize) / 3);
		int height = fontSize + 20;

		// Initialize graphics
		RectangleF rectF = new RectangleF(0, 0, width, height);
		Bitmap pic = new Bitmap(width, height, PixelFormat.Format24bppRgb);
		Graphics g = Graphics.FromImage(pic);
		g.InterpolationMode = InterpolationMode.HighQualityBicubic;
		g.SmoothingMode = SmoothingMode.HighQuality;
		g.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;

		// Set colors
		Color fontColor = Color.Black;
		Color rectColor = Color.White;
		SolidBrush fgBrush = new SolidBrush(fontColor);
		SolidBrush bgBrush = new SolidBrush(rectColor);

		// Rectangle or ellipse?
		g.FillRectangle(bgBrush, rectF);

		// Load font   
		FontFamily fontFamily = FontFamily.GenericSansSerif;
		FontStyle style = FontStyle.Regular;
		Font font = new Font(fontFamily, fontSize, style, GraphicsUnit.Pixel);

		// Set font direction & alignment
		StringFormat format = new StringFormat();
		format.Alignment = StringAlignment.Center;
		format.LineAlignment = StringAlignment.Center;

		// Finally, draw the font
		g.DrawString(text, font, fgBrush, rectF, format);
		context.Response.ContentType = "image/jpeg";
		pic.Save(context.Response.OutputStream, ImageFormat.Jpeg);

		// Dispose objects
		pic.Dispose();

		context.Response.End();
	}
}